@@ -83,6 +83,40 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
         STRINGOUT(MessageString);
 }
 
+//--------------------------------------------------------------------------
+void __stdcall Event_CallBackFunction(unsigned char* Data_Content, size_t Data_Size, void* UserHandle_Void)
+{
+    struct MediaInfo_Event_Generic*     Event_Generic=(struct MediaInfo_Event_Generic*)Data_Content;
+    unsigned short                      EventID;
+    unsigned char                       EventVersion;
+
+    /*integrity tests*/
+    if (Data_Size<4)
+        return; //There is a problem
+
+    /*Retrieving EventID*/
+    EventID     =(unsigned short)((Event_Generic->EventCode&0x00FFFF00)>>8 );
+    EventVersion=(unsigned char) ( Event_Generic->EventCode&0x000000FF     );
+
+    switch (EventID)
+    {
+        case MediaInfo_Event_Global_BytesRead:
+            if (EventVersion == 0 && Data_Size >= sizeof(struct MediaInfo_Event_Global_BytesRead_0) && UserHandle_Void != NULL)
+            {
+                struct MediaInfo_Event_Global_BytesRead_0* Event_Data=(struct MediaInfo_Event_Global_BytesRead_0*)Data_Content;
+                std::string* UserHandle=(std::string*)UserHandle_Void;
+
+                UserHandle->append((const char*)Event_Data->Content, Event_Data->Content_Size);
+            }
+            break;
+        case MediaInfo_Event_Log:
+            if (EventVersion == 0 && Data_Size >= sizeof(struct MediaInfo_Event_Log_0))
+                Log_0((struct MediaInfo_Event_Log_0*)Data_Content);
+            break;
+        default: ;
+    }
+}
+
     //**************************************************************************
     // CLI
     //**************************************************************************
@@ -117,6 +151,45 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
             if (files.empty())
                 return Help_Nothing();
 
+            // download remote policies
+            if (MCL.mil_has_curl_enabled())
+            {
+                for (size_t i = policies.size(); i > 0; --i)
+                {
+                    if (policies[i-1].find("fetch://")==0)
+                    {
+                        std::string policy = policies[i-1].substr(8);
+                        MediaInfoNameSpace::MediaInfo tmpMI;
+                        tmpMI.Option(__T("ParseSpeed"), __T("1"));
+
+                        for (size_t i = 0; i < options.size(); ++i)
+                        {
+                            std::string report;
+                           if (!options[i].first.empty() && !options[i].second.empty() && MCL.test_mil_option(options[i].first, options[i].second, report)==0)
+                               tmpMI.Option(ZenLib::Ztring().From_UTF8(options[i].first), ZenLib::Ztring().From_UTF8(options[i].second));
+                        }
+
+                        std::string data;
+                        std::stringstream ss;
+                        ss << "CallBack=memory://" << (ZenLib::int64u)Event_CallBackFunction << ";UserHandler=memory://" << (ZenLib::int64u)&data;
+                        tmpMI.Option(__T("File_Event_CallBackFunction"), ZenLib::Ztring().From_UTF8(ss.str().c_str()));
+                        tmpMI.Option(__T("Event_CallBackFunction"), ZenLib::Ztring().From_UTF8(ss.str().c_str()));
+                        tmpMI.Open(ZenLib::Ztring().From_UTF8(policy));
+
+                        if (!data.empty())
+                            policies[i-1] = data;
+                        else
+                        {
+                            err = "Unable to fetch policy file: " + policy + ".";
+                            return CLI_RETURN_ERROR;
+                        }
+
+                         tmpMI.Close();
+                         tmpMI.Option(__T("ParseSpeed"), __T("0.5"));
+                    }
+                }
+            }
+
             // If no report selected, use Implementation by default
             if (!report_set.count() && !policies.size())
                 report_set.set(MediaConchLib::report_MediaConch);
@@ -149,6 +222,44 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
                 format = MediaConchLib::format_Simple;
             else if (format == MediaConchLib::format_Max && !display_content.empty())
                 format = MediaConchLib::format_Xml;
+
+            for (size_t i = 0; i < outputs.size(); ++i)
+            {
+                MediaConchLib::format format = outputs[i].first;
+                std::string filename = outputs[i].second;
+
+                if (format == MediaConchLib::format_Max && display_content.empty())
+                {
+                    // Try to autodetect wanted output format
+                    if (filename.size() >= 4 && filename.rfind(".txt") == filename.size() - 4)
+                        format = MediaConchLib::format_Text;
+                    else if (filename.size() >= 4 && filename.rfind(".xml") == filename.size() - 4)
+                        format = MediaConchLib::format_Xml;
+                    else if (filename.size() >= 5 && filename.rfind(".html") == filename.size() - 5)
+                        format = MediaConchLib::format_Html;
+                    else if (filename.size() >= 4 && filename.rfind(".csv") == filename.size() - 4)
+                        format = MediaConchLib::format_CSV;
+                    else
+                        format = MediaConchLib::format_Simple;
+
+                    outputs[i].first = format;
+                }
+
+                if (format == MediaConchLib::format_CSV && !policies.size())
+                {
+                     err = "CLI can use CSV format only with policies.";
+                     return CLI_RETURN_ERROR;
+                }
+
+                if (!MCL.ReportAndFormatCombination_IsValid(files, report_set,
+                                                            format == MediaConchLib::format_Max ? display_content : std::string(),
+                                                            format, err))
+                {
+                    if (err == "MicroMediaTrace requires an XML output.")
+                        err += " Use only --output-xml=";
+                    return CLI_RETURN_ERROR;
+                }
+            }
         }
 
         MCL.set_configuration_file(configuration_file);
@@ -199,16 +310,32 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
 
         //Return plugins list
         if (plugins_list_mode)
-            return run_plugins_list(err);
+        {
+            if (run_plugins_list(err) < 0)
+               return CLI_RETURN_ERROR;
+            return CLI_RETURN_NONE;
+        }
         //Return watch folders list
         else if (list_watch_folders_mode)
-            return run_watch_folders_list(err);
+        {
+            if (run_watch_folders_list(err) < 0)
+               return CLI_RETURN_ERROR;
+            return CLI_RETURN_NONE;
+        }
         //Return list files
         else if (list_mode)
-            return run_list_files(err);
+        {
+            if (run_list_files(err) < 0)
+               return CLI_RETURN_ERROR;
+            return CLI_RETURN_NONE;
+        }
         // Add a watch folder
         else if (watch_folder.size())
-            return run_watch_folder_cmd(err);
+        {
+            if (run_watch_folder_cmd(err) < 0)
+               return CLI_RETURN_ERROR;
+            return CLI_RETURN_NONE;
+        }
 
         //Return file information
         if (file_information)
@@ -222,7 +349,7 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
             int ret = MCL.checker_analyze(use_as_user, files[i], plugins, options, registered,
                                           file_id, err, force_analyze, mil_analyze);
             if (ret < 0)
-                return -1;
+                return CLI_RETURN_ERROR;
 
             if (use_daemon && asynchronous && !registered)
             {
@@ -238,7 +365,7 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
                 continue;
             else if (ready < 0)
                 //TODO: PROBLEM
-                return -1;
+                return CLI_RETURN_ERROR;
 
             if (report_kind == MediaConchLib::report_Max)
                 report_kind = MediaConchLib::report_MediaConch;
@@ -267,40 +394,82 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
             int ret = MCL.checker_analyze(use_as_user, policy_reference_file, plugins, this->options, registered,
                                           file_id, err, force_analyze, mil_analyze);
             if (ret < 0)
-                return -1;
+                return CLI_RETURN_ERROR;
 
             if (run_policy_reference_file(file_id, err) < 0)
-                return -1;
+                return CLI_RETURN_ERROR;
 
             std::stringstream ss;
             ss << file_id;
             cr.options["policy_reference_id"] = ss.str();
         }
 
+        if (policy_verbosity.size())
+            cr.options["policy_verbosity"] = policy_verbosity;
+
+        if (full_report)
+            cr.options["full_report"] = "1";
+
         //Output
-        MediaConchLib::Checker_ReportRes result;
         cr.user = use_as_user;
         cr.report_set = report_set;
-        cr.format = format;
         cr.options["verbosity"] = MCL.get_implementation_verbosity();
 
         for (size_t i = 0; i < policies.size(); ++i)
             cr.policies_contents.push_back(policies[i]);
 
-        cr.display_content = &display_content;
-
         if (report_set[MediaConchLib::report_MediaInfo] && mi_inform.size())
             cr.mi_inform = &mi_inform;
 
-        MCL.checker_get_report(cr, &result, error);
-        MediaInfoLib::String report_mi = ZenLib::Ztring().From_UTF8(result.report);
+        if (outputs.empty()) //Print report to screen
+        {
+            MediaConchLib::Checker_ReportRes result;
+            cr.format = format;
 
-        STRINGOUT(report_mi);
-        //Output, in a file if needed
-        if (!LogFile_FileName.empty())
-            LogFile_Action(report_mi);
+            cr.display_content = &display_content;
 
-        return 0;
+            MCL.checker_get_report(cr, &result, error);
+            MediaInfoLib::String report_mi = ZenLib::Ztring().From_UTF8(result.report);
+
+            STRINGOUT(report_mi);
+            //Output, in a file if needed
+            if (!LogFile_FileName.empty())
+                LogFile_Action(report_mi);
+        }
+        else //Write report to file(s)
+        {
+            for (size_t i = 0; i < outputs.size(); ++i)
+            {
+                MediaConchLib::Checker_ReportRes result;
+
+                MediaConchLib::format format = outputs[i].first;
+                std::string filename = outputs[i].second;
+
+                cr.display_content = NULL;
+                if (format == MediaConchLib::format_Max)
+                {
+                    format = MediaConchLib::format_Xml; //Needed with display
+                    cr.display_content = &display_content;
+                }
+
+                cr.format = format;
+
+                MCL.checker_get_report(cr, &result, error);
+                MediaInfoLib::String report_mi = ZenLib::Ztring().From_UTF8(result.report);
+
+                ZenLib::File F;
+                if (!F.Create(ZenLib::Ztring().From_UTF8(outputs[i].second.c_str())))
+                {
+                    STRINGOUT(__T("Unable to write to output file: ") + ZenLib::Ztring().From_UTF8(filename));
+                    return CLI_RETURN_ERROR;
+                }
+
+                //Filling
+                F.Write(report_mi);
+            }
+        }
+
+        return CLI_RETURN_NONE;
     }
 
     //--------------------------------------------------------------------------
@@ -309,16 +478,16 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
         if (files_ids.size() != 1)
         {
             error = "Create a policy only with one file";
-            return MediaConchLib::errorHttp_INTERNAL;
+            return -1;
         }
 
         size_t pos = MCL.xslt_policy_create_from_file(use_as_user, files_ids[0], error);
         if (pos == (size_t)-1)
-            return MediaConchLib::errorHttp_INTERNAL;
+            return -1;
 
         std::string policy;
-        if (MCL.policy_dump(use_as_user, pos, false, policy, error))
-            return MediaConchLib::errorHttp_INTERNAL;
+        if (MCL.policy_dump(use_as_user, pos, false, policy, error) < 0)
+            return -1;
 
         MediaInfoLib::String policy_mil = ZenLib::Ztring().From_UTF8(policy);
         STRINGOUT(policy_mil);
@@ -498,6 +667,12 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
     //--------------------------------------------------------------------------
     int CLI::add_policy(const std::string& pattern)
     {
+        if (pattern.find("://")!=std::string::npos && MCL.mil_has_curl_enabled())
+        {
+            policies.push_back("fetch://" + pattern);
+            return 0;
+        }
+
         std::vector<std::string> filenames;
 #ifdef HAVE_GLOB
         // Policy filenames pattern matching
@@ -519,13 +694,19 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
             std::string& filename = filenames[i];
             ZenLib::Ztring z_filename = ZenLib::Ztring().From_UTF8(filename);
             if (!ZenLib::File::Exists(z_filename))
-                return -1;
+            {
+               STRINGERR(__T("Policy file: ") + z_filename + __T(" not found."));
+               return CLI_RETURN_ERROR;
+            }
 
             ZenLib::File file(z_filename);
 
             ZenLib::int64u size = file.Size_Get();
             if (size == (ZenLib::int64u)-1)
-                return -1;
+            {
+               STRINGERR(__T("Unable to open policy file: ") + z_filename + __T("."));
+               return CLI_RETURN_ERROR;
+            }
 
             ZenLib::int8u* Buffer = new ZenLib::int8u[size + 1];
 
@@ -599,6 +780,18 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
         return 0;
     }
 
+    //--------------------------------------------------------------------------
+    void CLI::set_policy_verbosity(const std::string& verbosity)
+    {
+        policy_verbosity = verbosity;
+    }
+
+    //--------------------------------------------------------------------------
+    void CLI::set_full_report(bool full)
+    {
+        full_report = full;
+    }
+
     //--------------------------------------------------------------------------
     int CLI::add_plugin_to_use(const std::string& plugin)
     {
@@ -698,6 +891,31 @@ void Log_0(struct MediaInfo_Event_Log_0* Event)
         no_needs_files_mode = true;
     }
 
+    //--------------------------------------------------------------------------
+    int  CLI::add_output(const std::string& format, const std::string& filename)
+    {
+        MediaConchLib::format f;
+        if (format == "text")
+            f = MediaConchLib::format_Text;
+        else if (format == "xml")
+            f = MediaConchLib::format_Xml;
+        else if (format == "maxml")
+            f = MediaConchLib::format_MaXml;
+        else if (format == "jstree")
+            f = MediaConchLib::format_JsTree;
+        else if (format == "html")
+            f = MediaConchLib::format_Html;
+        else if (format == "csv")
+            f = MediaConchLib::format_CSV;
+        else if (format == "simple")
+            f = MediaConchLib::format_Simple;
+        else
+            f = MediaConchLib::format_Max;
+
+        outputs.push_back(std::make_pair(f, filename));
+        return 0;
+    }
+
     //--------------------------------------------------------------------------
     int CLI::register_option(const std::string& key, std::string& value)
     {
