@@ -28,7 +28,7 @@ namespace MediaConch {
 // SQLLiteUi
 //***************************************************************************
 
-int SQLLiteUi::ui_current_version          = 12;
+int SQLLiteUi::ui_current_version          = 15;
 
 //***************************************************************************
 // Constructor/Destructor
@@ -112,6 +112,9 @@ int SQLLiteUi::ui_update_table()
     // UPDATE_UI_TABLE_FOR_VERSION(9); nothing to do
     UPDATE_UI_TABLE_FOR_VERSION(10);
     UPDATE_UI_TABLE_FOR_VERSION(11);
+    UPDATE_UI_TABLE_FOR_VERSION(12);
+    // UPDATE_UI_TABLE_FOR_VERSION(13); nothing to do
+    UPDATE_UI_TABLE_FOR_VERSION(14);
 
 #undef UPDATE_UI_TABLE_FOR_VERSION
 
@@ -140,8 +143,8 @@ int SQLLiteUi::ui_add_file(const FileRegistered* file)
 
     reports.clear();
     create << "INSERT INTO UI ";
-    create << "(FILENAME, FILEPATH, POLICY, DISPLAY, ANALYZED, IMPLEMENTATION_VALID, POLICY_VALID, VERBOSITY, OPTIONS, MIL_VERSION)";
-    create << " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?);";
+    create << "(FILENAME, FILEPATH, POLICY, DISPLAY, ANALYZED, IMPLEMENTATION_VALID, IMPLEMENTATION_HAS_INFO, IMPLEMENTATION_HAS_WARNING, POLICY_VALID, POLICY_HAS_INFO, POLICY_HAS_WARNING, VERBOSITY, OPTIONS, MIL_VERSION)";
+    create << " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);";
     query = create.str();
 
     const char* end = NULL;
@@ -173,25 +176,37 @@ int SQLLiteUi::ui_add_file(const FileRegistered* file)
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_int(stmt, 7, file->policy_valid);
+    ret = sqlite3_bind_int(stmt, 7, file->implementation_has_info);
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_int(stmt, 8, file->verbosity);
+    ret = sqlite3_bind_int(stmt, 8, file->implementation_has_warning);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 9, file->policy_valid);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 10, file->policy_has_info);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 11, file->policy_has_warning);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 12, file->verbosity);
     if (ret != SQLITE_OK)
         return -1;
 
     std::string options;
     options_vec_to_string(file->options, options);
-    ret = sqlite3_bind_blob(stmt, 9, options.c_str(), options.length(), SQLITE_TRANSIENT);
+    ret = sqlite3_bind_blob(stmt, 13, options.c_str(), options.length(), SQLITE_TRANSIENT);
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_int(stmt, 8, file->verbosity);
-    if (ret != SQLITE_OK)
-        return -1;
-
-    ret = sqlite3_bind_int64(stmt, 10, (sqlite3_int64)file->mil_version);
+    ret = sqlite3_bind_int64(stmt, 14, (sqlite3_int64)file->mil_version);
     if (ret != SQLITE_OK)
         return -1;
 
@@ -228,12 +243,12 @@ int SQLLiteUi::ui_create_files(const std::vector<FileRegistered*>& files)
 
     reports.clear();
     create << "INSERT INTO UI ";
-    create << "(FILENAME, FILEPATH, POLICY, DISPLAY, ANALYZED, IMPLEMENTATION_VALID, POLICY_VALID, VERBOSITY, OPTIONS, MIL_VERSION) VALUES";
+    create << "(FILENAME, FILEPATH, POLICY, DISPLAY, ANALYZED, IMPLEMENTATION_VALID, IMPLEMENTATION_HAS_INFO, IMPLEMENTATION_HAS_WARNING, POLICY_VALID, POLICY_HAS_INFO, POLICY_HAS_WARNING, VERBOSITY, OPTIONS, MIL_VERSION) VALUES";
     for (size_t i = 0; i < files.size(); ++i)
     {
         if (i)
             create << ",";
-        create << " (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
+        create << " (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
     }
 
     create << ";";
@@ -244,7 +259,7 @@ int SQLLiteUi::ui_create_files(const std::vector<FileRegistered*>& files)
     if (prepare_v2(query, err) < 0)
         return -1;
 
-    int nb_column = 10;
+    int nb_column = 12;
     int ret = SQLITE_OK;
     for (size_t i = 0; i < files.size(); ++i)
     {
@@ -273,22 +288,38 @@ int SQLLiteUi::ui_create_files(const std::vector<FileRegistered*>& files)
         if (ret != SQLITE_OK)
             return -1;
 
-        ret = sqlite3_bind_int(stmt, i * nb_column + 7, file->policy_valid);
+        ret = sqlite3_bind_int(stmt, i * nb_column + 7, file->implementation_has_info);
         if (ret != SQLITE_OK)
             return -1;
 
-        ret = sqlite3_bind_int(stmt, i * nb_column + 8, file->verbosity);
+        ret = sqlite3_bind_int(stmt, i * nb_column + 8, file->implementation_has_warning);
+        if (ret != SQLITE_OK)
+            return -1;
+
+        ret = sqlite3_bind_int(stmt, i * nb_column + 9, file->policy_valid);
+        if (ret != SQLITE_OK)
+            return -1;
+
+        ret = sqlite3_bind_int(stmt, i * nb_column + 10, file->policy_has_info);
+        if (ret != SQLITE_OK)
+            return -1;
+
+        ret = sqlite3_bind_int(stmt, i * nb_column + 11, file->policy_has_warning);
+        if (ret != SQLITE_OK)
+            return -1;
+
+        ret = sqlite3_bind_int(stmt, i * nb_column + 12, file->verbosity);
         if (ret != SQLITE_OK)
             return -1;
 
         std::string options;
         options_vec_to_string(file->options, options);
 
-        ret = sqlite3_bind_blob(stmt, i * nb_column + 9, options.c_str(), options.length(), SQLITE_TRANSIENT);
+        ret = sqlite3_bind_blob(stmt, i * nb_column + 13, options.c_str(), options.length(), SQLITE_TRANSIENT);
         if (ret != SQLITE_OK)
             return -1;
 
-        ret = sqlite3_bind_int64(stmt, i * nb_column + 10, (sqlite3_int64)file->mil_version);
+        ret = sqlite3_bind_int64(stmt, i * nb_column + 14, (sqlite3_int64)file->mil_version);
         if (ret != SQLITE_OK)
             return -1;
 
@@ -304,16 +335,20 @@ int SQLLiteUi::ui_update_file(const FileRegistered* file)
 
     reports.clear();
     create << "UPDATE UI ";
-    create << "SET POLICY           = ?, ";
-    create << "DISPLAY              = ?, ";
-    create << "ANALYZED             = ?, ";
-    create << "IMPLEMENTATION_VALID = ?, ";
-    create << "POLICY_VALID         = ?, ";
-    create << "VERBOSITY            = ?, ";
-    create << "OPTIONS              = ?, ";
-    create << "MIL_VERSION          = ?  ";
-    create << "WHERE FILENAME       = ?  ";
-    create << "AND   FILEPATH       = ? ;";
+    create << "SET POLICY                 = ?, ";
+    create << "DISPLAY                    = ?, ";
+    create << "ANALYZED                   = ?, ";
+    create << "IMPLEMENTATION_VALID       = ?, ";
+    create << "IMPLEMENTATION_HAS_INFO    = ?, ";
+    create << "IMPLEMENTATION_HAS_WARNING = ?, ";
+    create << "POLICY_VALID               = ?, ";
+    create << "POLICY_HAS_INFO            = ?, ";
+    create << "POLICY_HAS_WARNING         = ?, ";
+    create << "VERBOSITY                  = ?, ";
+    create << "OPTIONS                    = ?, ";
+    create << "MIL_VERSION                = ?  ";
+    create << "WHERE FILENAME             = ?  ";
+    create << "AND   FILEPATH             = ? ;";
 
     query = create.str();
 
@@ -338,29 +373,45 @@ int SQLLiteUi::ui_update_file(const FileRegistered* file)
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_int(stmt, 5, file->policy_valid);
+    ret = sqlite3_bind_int(stmt, 5, file->implementation_has_info);
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_int(stmt, 6, file->verbosity);
+    ret = sqlite3_bind_int(stmt, 6, file->implementation_has_warning);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 7, file->policy_valid);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 8, file->policy_has_info);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 9, file->policy_has_warning);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 10, file->verbosity);
     if (ret != SQLITE_OK)
         return -1;
 
     std::string options;
     options_vec_to_string(file->options, options);
-    ret = sqlite3_bind_blob(stmt, 7, options.c_str(), options.length(), SQLITE_TRANSIENT);
+    ret = sqlite3_bind_blob(stmt, 11, options.c_str(), options.length(), SQLITE_TRANSIENT);
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_int64(stmt, 8, (sqlite3_int64)file->mil_version);
+    ret = sqlite3_bind_int64(stmt, 12, (sqlite3_int64)file->mil_version);
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_blob(stmt, 9, file->filename.c_str(), file->filename.length(), SQLITE_STATIC);
+    ret = sqlite3_bind_blob(stmt, 13, file->filename.c_str(), file->filename.length(), SQLITE_STATIC);
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_blob(stmt, 10, file->filepath.c_str(), file->filepath.length(), SQLITE_STATIC);
+    ret = sqlite3_bind_blob(stmt, 14, file->filepath.c_str(), file->filepath.length(), SQLITE_STATIC);
     if (ret != SQLITE_OK)
         return -1;
 
@@ -373,13 +424,13 @@ int SQLLiteUi::ui_update_files(const std::vector<FileRegistered*>& files)
     std::stringstream create;
 
     reports.clear();
-    create << "WITH Tmp(FILENAME, FILEPATH, POLICY, DISPLAY, ANALYZED, IMPLEMENTATION_VALID, POLICY_VALID, VERBOSITY, OPTIONS, MIL_VERSION)";
+    create << "WITH Tmp(FILENAME, FILEPATH, POLICY, DISPLAY, ANALYZED, IMPLEMENTATION_VALID, IMPLEMENTATION_HAS_INFO, IMPLEMENTATION_HAS_WARNING, POLICY_VALID, POLICY_HAS_INFO, POLICY_HAS_WARNING, VERBOSITY, OPTIONS, MIL_VERSION)";
     create << "AS (VALUES";
     for (size_t i = 0; i < files.size(); ++i)
     {
         if (i)
             create << ", ";
-        create << "(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
+        create << "(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
     }
     create << ") ";
 
@@ -388,7 +439,11 @@ int SQLLiteUi::ui_update_files(const std::vector<FileRegistered*>& files)
     create << "DISPLAY = (SELECT DISPLAY FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
     create << "ANALYZED = (SELECT ANALYZED FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
     create << "IMPLEMENTATION_VALID = (SELECT IMPLEMENTATION_VALID FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
+    create << "IMPLEMENTATION_HAS_INFO = (SELECT IMPLEMENTATION_HAS_INFO FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
+    create << "IMPLEMENTATION_HAS_WARNING = (SELECT IMPLEMENTATION_HAS_WARNING FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
     create << "POLICY_VALID = (SELECT POLICY_VALID FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
+    create << "POLICY_HAS_INFO = (SELECT POLICY_HAS_INFO FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
+    create << "POLICY_HAS_WARNING = (SELECT POLICY_HAS_WARNING FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
     create << "VERBOSITY = (SELECT VERBOSITY FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
     create << "OPTIONS = (SELECT OPTIONS FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH), ";
     create << "MIL_VERSION = (SELECT MIL_VERSION FROM Tmp WHERE UI.FILENAME = Tmp.FILENAME AND UI.FILEPATH = Tmp.FILEPATH) ";
@@ -402,7 +457,7 @@ int SQLLiteUi::ui_update_files(const std::vector<FileRegistered*>& files)
     if (ret != SQLITE_OK || !stmt || (end && *end))
         return -1;
 
-    int nb_column = 10;
+    int nb_column = 12;
     for (size_t i = 0; i < files.size(); ++i)
     {
         FileRegistered* file = files[i];
@@ -431,21 +486,38 @@ int SQLLiteUi::ui_update_files(const std::vector<FileRegistered*>& files)
         if (ret != SQLITE_OK)
             return -1;
 
-        ret = sqlite3_bind_int(stmt, i * nb_column + 7, file->policy_valid);
+        ret = sqlite3_bind_int(stmt, i * nb_column + 7, file->implementation_has_info);
         if (ret != SQLITE_OK)
             return -1;
 
-        ret = sqlite3_bind_int(stmt, i * nb_column + 8, file->verbosity);
+        ret = sqlite3_bind_int(stmt, i * nb_column + 8, file->implementation_has_warning);
+        if (ret != SQLITE_OK)
+            return -1;
+
+
+        ret = sqlite3_bind_int(stmt, i * nb_column + 9, file->policy_valid);
+        if (ret != SQLITE_OK)
+            return -1;
+
+        ret = sqlite3_bind_int(stmt, i * nb_column + 10, file->policy_has_info);
+        if (ret != SQLITE_OK)
+            return -1;
+
+        ret = sqlite3_bind_int(stmt, i * nb_column + 11, file->policy_has_warning);
+        if (ret != SQLITE_OK)
+            return -1;
+
+        ret = sqlite3_bind_int(stmt, i * nb_column + 12, file->verbosity);
         if (ret != SQLITE_OK)
             return -1;
 
         std::string options;
         options_vec_to_string(file->options, options);
-        ret = sqlite3_bind_blob(stmt, i * nb_column + 9, options.c_str(), options.length(), SQLITE_TRANSIENT);
+        ret = sqlite3_bind_blob(stmt, i * nb_column + 13, options.c_str(), options.length(), SQLITE_TRANSIENT);
         if (ret != SQLITE_OK)
             return -1;
 
-        ret = sqlite3_bind_int64(stmt, i * nb_column + 10, (sqlite3_int64)file->mil_version);
+        ret = sqlite3_bind_int64(stmt, i * nb_column + 14, (sqlite3_int64)file->mil_version);
         if (ret != SQLITE_OK)
             return -1;
     }
@@ -462,13 +534,17 @@ int SQLLiteUi::ui_get_file(FileRegistered* file)
     std::string d("DISPLAY");
     std::string a("ANALYZED");
     std::string i_v("IMPLEMENTATION_VALID");
+    std::string i_i("IMPLEMENTATION_HAS_INFO");
+    std::string i_w("IMPLEMENTATION_HAS_WARNING");
     std::string p_v("POLICY_VALID");
+    std::string p_i("POLICY_HAS_INFO");
+    std::string p_w("POLICY_HAS_WARNING");
     std::string v("VERBOSITY");
     std::string o("OPTIONS");
     std::string m_v("MIL_VERSION");
 
     reports.clear();
-    create << "SELECT " << p << ", " << d << ", " << a << ", " << i_v << ", " << p_v << ", " << v << ", " << o << ", " << m_v << " FROM UI ";
+    create << "SELECT " << p << ", " << d << ", " << a << ", " << i_v << ", " << i_i << ", " << i_w << ", " << p_v << ", "<< p_i << ", " << p_w << ", " << v << ", " << o << ", " << m_v << " FROM UI ";
     create << "WHERE FILENAME = ? AND FILEPATH = ?;";
     query = create.str();
 
@@ -498,8 +574,16 @@ int SQLLiteUi::ui_get_file(FileRegistered* file)
         file->analyzed = std_string_to_int(r[a]);
     if (r.find(i_v) != r.end())
         file->implementation_valid = std_string_to_int(r[i_v]);
+    if (r.find(i_i) != r.end())
+        file->implementation_has_info = std_string_to_int(r[i_i]);
+    if (r.find(i_w) != r.end())
+        file->implementation_has_warning = std_string_to_int(r[i_w]);
     if (r.find(p_v) != r.end())
         file->policy_valid = std_string_to_int(r[p_v]);
+    if (r.find(p_i) != r.end())
+        file->policy_has_info = std_string_to_int(r[p_i]);
+    if (r.find(p_w) != r.end())
+        file->policy_has_warning = std_string_to_int(r[p_w]);
     if (r.find(v) != r.end())
         file->verbosity = std_string_to_int(r[v]);
     if (r.find(o) != r.end())
@@ -637,14 +721,18 @@ void SQLLiteUi::ui_get_elements(std::vector<FileRegistered*>& vec)
     std::string d("DISPLAY");
     std::string a("ANALYZED");
     std::string i_v("IMPLEMENTATION_VALID");
+    std::string i_i("IMPLEMENTATION_HAS_INFO");
+    std::string i_w("IMPLEMENTATION_HAS_WARNING");
     std::string p_v("POLICY_VALID");
+    std::string p_i("POLICY_HAS_INFO");
+    std::string p_w("POLICY_HAS_WARNING");
     std::string v("VERBOSITY");
     std::string o("OPTIONS");
     std::string m_v("MIL_VERSION");
 
     reports.clear();
     create << "SELECT " << name << ", " << path << ", " << p << ", " << d << ", " << a ;
-    create << ", " << i_v << ", " << p_v << ", " << v << ", " << o << ", " << m_v;
+    create << ", " << i_v << ", " << p_v << ", " << p_i << ", " << i_i << ", " << i_w << ", " << p_w << ", " << v << ", " << o << ", " << m_v;
     create << " FROM UI;";
     query = create.str();
 
@@ -672,8 +760,16 @@ void SQLLiteUi::ui_get_elements(std::vector<FileRegistered*>& vec)
                 file->analyzed = std_string_to_int(reports[i][a]);
             if (reports[i].find(i_v) != reports[i].end())
                 file->implementation_valid = std_string_to_int(reports[i][i_v]);
+            if (reports[i].find(i_i) != reports[i].end())
+                file->implementation_has_info = std_string_to_int(reports[i][i_i]);
+            if (reports[i].find(i_w) != reports[i].end())
+                file->implementation_has_warning = std_string_to_int(reports[i][i_w]);
             if (reports[i].find(p_v) != reports[i].end())
                 file->policy_valid = std_string_to_int(reports[i][p_v]);
+            if (reports[i].find(p_i) != reports[i].end())
+                file->policy_has_info = std_string_to_int(reports[i][p_i]);
+            if (reports[i].find(p_w) != reports[i].end())
+                file->policy_has_warning = std_string_to_int(reports[i][p_w]);
             if (reports[i].find(v) != reports[i].end())
                 file->verbosity = std_string_to_int(reports[i][v]);
             if (reports[i].find(o) != reports[i].end())
@@ -694,11 +790,15 @@ int SQLLiteUi::ui_reset_file_for_reload(FileRegistered* file)
 
     reports.clear();
     create << "UPDATE UI ";
-    create << "SET ANALYZED         = ?, ";
-    create << "IMPLEMENTATION_VALID = ?, ";
-    create << "POLICY_VALID         = ?, ";
-    create << "WHERE FILENAME       = ?  ";
-    create << "AND   FILEPATH       = ? ;";
+    create << "SET ANALYZED               = ?, ";
+    create << "IMPLEMENTATION_VALID       = ?, ";
+    create << "IMPLEMENTATION_HAS_INFO    = ?, ";
+    create << "IMPLEMENTATION_HAS_WARNING = ?, ";
+    create << "POLICY_VALID               = ?, ";
+    create << "POLICY_HAS_INFO            = ?, ";
+    create << "POLICY_HAS_WARNING         = ?, ";
+    create << "WHERE FILENAME             = ?  ";
+    create << "AND   FILEPATH             = ? ;";
 
     query = create.str();
 
@@ -719,11 +819,28 @@ int SQLLiteUi::ui_reset_file_for_reload(FileRegistered* file)
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_blob(stmt, 4, file->filename.c_str(), file->filename.length(), SQLITE_STATIC);
+
+    ret = sqlite3_bind_int(stmt, 4, false);
     if (ret != SQLITE_OK)
         return -1;
 
-    ret = sqlite3_bind_blob(stmt, 5, file->filepath.c_str(), file->filepath.length(), SQLITE_STATIC);
+    ret = sqlite3_bind_int(stmt, 5, false);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 6, false);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 7, false);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_blob(stmt, 8, file->filename.c_str(), file->filename.length(), SQLITE_STATIC);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_blob(stmt, 9, file->filepath.c_str(), file->filepath.length(), SQLITE_STATIC);
     if (ret != SQLITE_OK)
         return -1;
 
@@ -791,7 +908,9 @@ int SQLLiteUi::ui_settings_update_table()
     UPDATE_UI_SETTINGS_TABLE_FOR_VERSION(9);
     // UPDATE_UI_SETTINGS_TABLE_FOR_VERSION(10); nothing to do
     // UPDATE_UI_SETTINGS_TABLE_FOR_VERSION(11); nothing to do
-
+    // UPDATE_UI_SETTINGS_TABLE_FOR_VERSION(12); nothing to do
+    UPDATE_UI_SETTINGS_TABLE_FOR_VERSION(13);
+    // UPDATE_UI_SETTINGS_TABLE_FOR_VERSION(14); nothing to do
 #undef UPDATE_UI_SETTINGS_TABLE_FOR_VERSION
 
     return 0;
@@ -1029,6 +1148,64 @@ int SQLLiteUi::ui_settings_get_default_verbosity(int& verbosity, int user_id)
     return 0;
 }
 
+//---------------------------------------------------------------------------
+int SQLLiteUi::ui_settings_save_default_parsespeed(const std::string& parsespeed, int user_id)
+{
+    if (ui_settings_check_user_id(user_id))
+        return -1;
+
+    std::stringstream create;
+
+    reports.clear();
+    create << "UPDATE UI_SETTINGS ";
+    create << "SET   DEFAULT_PARSESPEED = ? ";
+    create << "WHERE USER_ID         = ?;";
+
+    query = create.str();
+
+    const char* end = NULL;
+    int ret = sqlite3_prepare_v2(db, query.c_str(), query.length() + 1, &stmt, &end);
+    if (ret != SQLITE_OK || !stmt || (end && *end))
+        return -1;
+
+    ret = sqlite3_bind_blob(stmt, 1, parsespeed.c_str(), parsespeed.length(), SQLITE_STATIC);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 2, user_id);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    return execute();
+    return 0;
+}
+
+//---------------------------------------------------------------------------
+int SQLLiteUi::ui_settings_get_default_parsespeed(std::string& parsespeed, int user_id)
+{
+    if (ui_settings_check_user_id(user_id))
+        return -1;
+
+    reports.clear();
+    query = "SELECT DEFAULT_PARSESPEED FROM UI_SETTINGS WHERE USER_ID = ?;";
+
+    const char* end = NULL;
+    int ret = sqlite3_prepare_v2(db, query.c_str(), query.length() + 1, &stmt, &end);
+    if (ret != SQLITE_OK || !stmt || (end && *end))
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 1, user_id);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    if (execute() || reports.size() != 1)
+        return -1;
+
+    if (reports[0].find("DEFAULT_PARSESPEED") != reports[0].end())
+        parsespeed = reports[0]["DEFAULT_PARSESPEED"];
+    return 0;
+}
+
 //---------------------------------------------------------------------------
 int SQLLiteUi::ui_settings_save_last_policy(const std::string& policy, int user_id)
 {
@@ -1201,6 +1378,64 @@ int SQLLiteUi::ui_settings_get_last_verbosity(int& verbosity, int user_id)
     return 0;
 }
 
+//---------------------------------------------------------------------------
+int SQLLiteUi::ui_settings_save_last_parsespeed(const std::string& parsespeed, int user_id)
+{
+    if (ui_settings_check_user_id(user_id))
+        return -1;
+
+    std::stringstream create;
+
+    reports.clear();
+    create << "UPDATE UI_SETTINGS ";
+    create << "SET   LAST_PARSESPEED = ? ";
+    create << "WHERE USER_ID         = ?;";
+
+    query = create.str();
+
+    const char* end = NULL;
+    int ret = sqlite3_prepare_v2(db, query.c_str(), query.length() + 1, &stmt, &end);
+    if (ret != SQLITE_OK || !stmt || (end && *end))
+        return -1;
+
+    ret = sqlite3_bind_blob(stmt, 1, parsespeed.c_str(), parsespeed.length(), SQLITE_STATIC);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 2, user_id);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    return execute();
+    return 0;
+}
+
+//---------------------------------------------------------------------------
+int SQLLiteUi::ui_settings_get_last_parsespeed(std::string& parsespeed, int user_id)
+{
+    if (ui_settings_check_user_id(user_id))
+        return -1;
+
+    reports.clear();
+    query = "SELECT LAST_PARSESPEED FROM UI_SETTINGS WHERE USER_ID = ?;";
+
+    const char* end = NULL;
+    int ret = sqlite3_prepare_v2(db, query.c_str(), query.length() + 1, &stmt, &end);
+    if (ret != SQLITE_OK || !stmt || (end && *end))
+        return -1;
+
+    ret = sqlite3_bind_int(stmt, 1, user_id);
+    if (ret != SQLITE_OK)
+        return -1;
+
+    if (execute() || reports.size() != 1)
+        return -1;
+
+    if (reports[0].find("LAST_PARSESPEED") != reports[0].end())
+        parsespeed = reports[0]["LAST_PARSESPEED"];
+    return 0;
+}
+
 //---------------------------------------------------------------------------
 int SQLLiteUi::ui_settings_save_default_load_files_path(const std::string& load_path, int user_id)
 {
