@@ -451,6 +451,7 @@ namespace MediaConch
     void WebCommonPage::on_save_settings_selected(const QString& policy,
                                             const QString& display,
                                             const QString& verbosity,
+                                            const QString& parsespeed,
                                             const QString& save_report_path, const QString& load_files_path,
                                             const QString& save_policy_path, const QString& load_policy_path,
                                             const QString& save_display_path, const QString& load_display_path)
@@ -481,6 +482,12 @@ namespace MediaConch
         //verbosity
         settings.change_default_verbosity(verbosity.toInt());
 
+        //verbosity
+        if (parsespeed == "-1")
+            settings.change_default_parsespeed("last");
+        else
+            settings.change_default_parsespeed(parsespeed.toStdString());
+
         //Save report path
         std::string r_s_path;
         if (save_report_path == "last")
@@ -599,10 +606,12 @@ namespace MediaConch
         int display_i = mainwindow->select_correct_display();
         int verbosity_i = mainwindow->select_correct_verbosity();
         bool has_libcurl = mainwindow->mil_has_curl_enabled();
+        QString parsespeed = QString().fromStdString(mainwindow->select_correct_parsespeed());
 
         use_javascript(QString("document.getElementById('checkerUpload_policy').value = %1;").arg(policy_i));
         use_javascript(QString("document.getElementById('checkerUpload_display_selector').value = %1;").arg(display_i));
         use_javascript(QString("document.getElementById('checkerUpload_verbosity_selector').value = %1;").arg(verbosity_i));
+        use_javascript(QString("document.getElementById('checkerUpload_parsespeed_selector').value = \"%1\";").arg(parsespeed));
         use_javascript("document.getElementById('checkerUpload_file').value = \"\";");
 
         if (has_libcurl)
@@ -610,12 +619,14 @@ namespace MediaConch
             use_javascript(QString("document.getElementById('checkerOnline_policy').value = %1;").arg(policy_i));
             use_javascript(QString("document.getElementById('checkerOnline_display_selector').value = %1;").arg(display_i));
             use_javascript(QString("document.getElementById('checkerOnline_verbosity_selector').value = %1;").arg(verbosity_i));
+            use_javascript(QString("document.getElementById('checkerOnline_parsespeed_selector').value = \"%1\";").arg(parsespeed));
             use_javascript("document.getElementById('checkerOnline_file').value = \"\";");
         }
 
         use_javascript(QString("document.getElementById('checkerRepository_policy').value = %1;").arg(policy_i));
         use_javascript(QString("document.getElementById('checkerRepository_display_selector').value = %1;").arg(display_i));
         use_javascript(QString("document.getElementById('checkerRepository_verbosity_selector').value = %1;").arg(verbosity_i));
+        use_javascript(QString("document.getElementById('checkerRepository_parsespeed_selector').value = \"%1\";").arg(parsespeed));
         use_javascript("document.getElementById('checkerRepository_directory').value = \"\";");
     }
 
@@ -747,10 +758,10 @@ namespace MediaConch
         }
 
         QStringList options;
-        if (checker_full_parse)
+        if (!checker_parse_speed.isEmpty())
         {
             options << "file_parsespeed";
-            options << "1";
+            options << checker_parse_speed;
         }
         QString res = on_file_upload_selected(QString().setNum(mainwindow->select_correct_policy()),
                                               QString().setNum(mainwindow->select_correct_display()),
@@ -850,25 +861,25 @@ namespace MediaConch
 
     QString WebCommonPage::policy_is_valid(long file_id, long policy_id)
     {
-        //{"valid":false,"fileId":"fileId","error":null}
+        //{"valid":false,"info":false,"warn":false,"fileId":"fileId","error":null}
         std::string err;
         QString json = QString("{\"fileId\":\"%1\",").arg(file_id);
         bool policy_valid = false;
         if (mainwindow->update_policy_of_file_in_list(file_id, policy_id, err) < 0)
         {
-            json += QString("\"valid\":false,\"error\":\"%2\"}")
+            json += QString("\"valid\":false,\"info\":false,\"warn\":false,\"error\":\"%2\"}")
                 .arg(QString().fromUtf8(err.c_str(), err.size()));
         }
 
         FileRegistered* fr = mainwindow->get_file_registered_from_id(file_id);
         if (!fr)
         {
-            json += QString("\"valid\":false,\"error\":\"%2\"}")
+            json += QString("\"valid\":false,\"info\":false,\"warn\":false,\"error\":\"%2\"}")
                 .arg(QString().fromUtf8("File has been removed."));
         }
 
-        json += QString("\"valid\":%1,\"error\":%2}")
-            .arg(fr->policy_valid ? "true" : "false").arg("null");
+        json += QString("\"valid\":%1,\"info\":%2,\"warn\":%3,\"error\":%4}")
+            .arg(fr->policy_valid ? "true" : "false").arg(fr->policy_has_info ? "true" : "false").arg(fr->policy_has_warning ? "true" : "false").arg("null");
 
         delete fr;
         return json;
@@ -881,21 +892,24 @@ namespace MediaConch
         FileRegistered* fr = mainwindow->get_file_registered_from_id(file_id);
         if (!fr)
         {
-            json += QString("\"valid\":%1,\"error\":\"%2\"}")
-                .arg("false").arg("File not reachable");
+            json += QString("\"valid\":%1,\"info\":%2,\"warn\":%3,\"error\":\"%4\"}")
+                .arg("false").arg("false").arg("false").arg("File not reachable");
             return json;
         }
 
         if (!fr->analyzed)
         {
-            json += QString("\"valid\":%1,\"error\":\"%2\"}")
-                .arg("false").arg("File not analyzed");
+            json += QString("\"valid\":%1,\"info\":%2,\"warn\":%3,\"error\":\"%4\"}")
+                .arg("false").arg("false").arg("false").arg("File not analyzed");
             delete fr;
             return json;
         }
 
-        json += QString("\"valid\":%1,\"error\":%2}")
-            .arg(fr->implementation_valid ? "true" : "false").arg("null");
+        json += QString("\"valid\":%1,\"info\":%2,\"warn\":%3,\"error\":%4}")
+            .arg(fr->implementation_valid ? "true" : "false")
+            .arg(fr->implementation_has_info ? "true" : "false")
+            .arg(fr->implementation_has_warning ? "true" : "false")
+            .arg("null");
 
         delete fr;
         return json;
@@ -1054,10 +1068,15 @@ namespace MediaConch
         else
             rule_data += ",\"field\":\"\"";
 
-        if (r->occurrence >= 0)
-            rule_data += QString(",\"occurrence\":%1").arg(r->occurrence);
+        len = r->occurrence.length();
+        if (len > 0)
+        {
+            QString occurrence = QString().fromUtf8(r->occurrence.c_str(), r->occurrence.length());
+            string_to_json(occurrence);
+            rule_data += QString(",\"occurrence\":\"%1\"").arg(occurrence);
+        }
         else
-            rule_data += ",\"occurrence\":\"-1\"";
+            rule_data += ",\"occurrence\":\"*\"";
 
         len = r->ope.length();
         if (len > 0)
@@ -1089,6 +1108,16 @@ namespace MediaConch
         else
             rule_data += ",\"scope\":\"\"";
 
+        len = r->level.length();
+        if (len > 0)
+        {
+            QString level = QString().fromUtf8(r->level.c_str(), r->level.length());
+            string_to_json(level);
+            rule_data += QString(",\"level\":\"%1\"").arg(level);
+        }
+        else
+            rule_data += ",\"level\":\"\"";
+
         rule_data += "}";
     }
 
@@ -1406,7 +1435,7 @@ namespace MediaConch
     }
 
     QString WebCommonPage::policy_edit(int id, const QString& name, const QString& description, const QStringList& tags, const QString& license,
-                                       const QString& type, const QString&)
+                                       const QString& type, const QString& level, const QString&)
     {
         //return: error?
         QString json;
@@ -1418,7 +1447,7 @@ namespace MediaConch
             tgs.push_back(tags[i].toUtf8().data());
 
         if ((code = mainwindow->policy_change_info((size_t)id, name.toUtf8().data(), description.toUtf8().data(),
-                                                   tgs, license.toUtf8().data(), err)) < 0)
+                                                   tgs, level.toUtf8().data(), license.toUtf8().data(), err)) < 0)
         {
             string_to_json(err);
             json = QString("{\"error\":\"%1\"}").arg(err);
@@ -1492,7 +1521,7 @@ namespace MediaConch
         return json;
     }
 
-    QString WebCommonPage::xslt_policy_rule_edit(int rule_id, int policy_id, const QString& title, const QString& type, const QString& field, int occurrence, const QString& ope, const QString& value, const QString& scope)
+    QString WebCommonPage::xslt_policy_rule_edit(int rule_id, int policy_id, const QString& title, const QString& type, const QString& field, const QString& occurrence, const QString& ope, const QString& value, const QString& scope, const QString& level)
     {
         QString err;
         QString json;
@@ -1502,9 +1531,10 @@ namespace MediaConch
         rule.ope           = ope.toUtf8().data();
         rule.track_type    = type.toUtf8().data();
         rule.field         = field.toUtf8().data();
-        rule.occurrence    = occurrence;
+        rule.occurrence    = occurrence.toUtf8().data();
         rule.value         = value.toUtf8().data();
         rule.scope         = scope.toUtf8().data();
+        rule.level         = level.toUtf8().data();
 
         int code;
         if ((code = mainwindow->xslt_policy_rule_edit(policy_id, rule_id, &rule, err)) < 0)
@@ -1784,8 +1814,8 @@ namespace MediaConch
         return QString("{}");
     }
 
-    void WebCommonPage::set_full_parse(bool fp)
+    void WebCommonPage::set_parse_speed(const QString& parse_speed)
     {
-        checker_full_parse = fp;
+        checker_parse_speed = parse_speed;
     }
 }
